//+------------------------------------------------------------------+
//|                                                BBands of CCI.mq4 |
//| Bollinger Bands of CCI                                           |
//|                               Copyright  2011, PDSoftware Corp. |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, PDSoftware Corp."
#property link      "http://www.metaquotes.net"
//#property strict

//----

#property indicator_separate_window
#property indicator_buffers 8

#property indicator_color1 Black
#property indicator_color2 Black
#property indicator_color3 DodgerBlue
#property indicator_color4 LimeGreen
#property indicator_color5 Red


//---- indicator parameters

extern string MAIN_PARAMS;                   // MAIN INDICATOR SETTINGS
extern int    BandsPeriod=20;                // Period of bollinger bands
extern int    BandsShift=0;                  // Shift of bollinger bands
extern double BandsDeviations=2.0;           // Deviation of bollinger bands 
extern int    CCIPeriod = 14;                // Period of CCI
extern string STYLES_PARAMS;                 // INDICATOR LINES STYLES
extern int    BandsLinesWidth = 1;           // Width of bollinger bands lines
extern int    MdBandsLinesWidth = 1;         // Width of median line
extern int    CCIlinesWidth = 2;             // Width of CCI line
extern string COLORS_PARAMS;                 // INDICATOR COLORS
extern color  UpBandColor = Black;           // Upper bollinger band color
extern color  DnBandColor = Black;           // Lower bollinger band color
extern color  MdLineColor = DodgerBlue;      // Median line color
extern color  CCIUpColor = LimeGreen;        // CCI cross up color
extern color  CCIDnColor = Red;              // CCI cross down color

//---- buffers
double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];
double CCIBuffer[];
double cciUa[];
double cciUb[];
double cciDa[];
double cciDb[];
double value[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- Set indicators
   SetIndexBuffer(0,MovingBuffer); SetIndexStyle(0,DRAW_LINE,0,MdBandsLinesWidth,MdLineColor);  
   SetIndexBuffer(1,UpperBuffer);  SetIndexStyle(1,DRAW_LINE,0,BandsLinesWidth,UpBandColor);  
   SetIndexBuffer(2,LowerBuffer);  SetIndexStyle(2,DRAW_LINE,0,BandsLinesWidth,DnBandColor);
   SetIndexBuffer(3,CCIBuffer);    SetIndexStyle(3,DRAW_NONE);  
 //  SetIndexBuffer(4,cciUp);        SetIndexStyle(4,DRAW_LINE,CCIlinesWidth,CCIUpColor);
  // SetIndexBuffer(5,cciDn);        SetIndexStyle(5,DRAW_LINE,CCIlinesWidth,CCIDnColor);
   
   SetIndexBuffer(4, cciUa);   SetIndexStyle(4,EMPTY,EMPTY,CCIlinesWidth,CCIUpColor);
   SetIndexBuffer(5, cciUb);   SetIndexStyle(5,EMPTY,EMPTY,CCIlinesWidth,CCIUpColor);
   SetIndexBuffer(6, cciDa);   SetIndexStyle(6,EMPTY,EMPTY,CCIlinesWidth,CCIDnColor);
   SetIndexBuffer(7, cciDb);   SetIndexStyle(7,EMPTY,EMPTY,CCIlinesWidth,CCIDnColor);
   
   
//----
   SetIndexDrawBegin(0,BandsPeriod+BandsShift);
   SetIndexDrawBegin(1,BandsPeriod+BandsShift);
   SetIndexDrawBegin(2,BandsPeriod+BandsShift);
   SetIndexDrawBegin(3,CCIPeriod);
   SetIndexDrawBegin(4,CCIPeriod);
   SetIndexDrawBegin(5,CCIPeriod);
   
//----
   SetLevelValue(0,100);
   SetLevelValue(1,-100);
//----
   SetIndexLabel(0,"SMA");
   SetIndexLabel(1,"Upper");
   SetIndexLabel(2,"Lower");
   SetIndexLabel(3,"CCI");

//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("BBands of CCI ("+BandsPeriod+", "+DoubleToStr(BandsDeviations,1)+", "+CCIPeriod+")");
   
   return(0);
  }
//+------------------------------------------------------------------+
//| Bollinger Bands of CCI                                           |
//+------------------------------------------------------------------+
int start()

  { 
   int    i,k,counted_bars=IndicatorCounted();
   double deviation;
   double sum,oldval,newres;
//----
   if(Bars<=BandsPeriod) return(0);
//---- initial zero
   if(counted_bars<1)
      for(i=1;i<=BandsPeriod;i++)
        {
         MovingBuffer[Bars-i]=EMPTY_VALUE;
         UpperBuffer[Bars-i]=EMPTY_VALUE;
         LowerBuffer[Bars-i]=EMPTY_VALUE;
        }
//----
   int limit=Bars-counted_bars;
   if(counted_bars>0) limit++;
   
   //Calculate CCI
  // for(i=0; i<limit; i++)
   for(i=limit; i>=0; i--)
      CCIBuffer[i]= iCCI(NULL,0,CCIPeriod,PRICE_TYPICAL,i);
      cciDa[i] = EMPTY_VALUE;
      cciDb[i] = EMPTY_VALUE;
      cciUa[i] = EMPTY_VALUE;
      cciUb[i] = EMPTY_VALUE;
      
       if (i<Bars-1)
          {   
            value[i] = value[i+1];
      
      if (CCIBuffer[i]>MovingBuffer[i])  value[i] =  1;
      if (CCIBuffer[i]<MovingBuffer[i])  value[i] = -1;
         
      
      if (value[i]== 1) PlotPoint(i,cciUa,cciUb,CCIBuffer);
      if (value[i]==-1) PlotPoint(i,cciDa,cciDb,CCIBuffer);
      
   //Calculate MA of CCI x BBands
   for(i=0; i<limit; i++)
      MovingBuffer[i]=iMAOnArray(CCIBuffer,Bars,BandsPeriod,0,MODE_SMA,i);
      

//----
   i=Bars-BandsPeriod+1;
   if(counted_bars>BandsPeriod-1) i=Bars-counted_bars-1;

   //Calculate Upper & Lower Bands
   while(i>=0)
     {
      sum=0.0;
      k=i+BandsPeriod-1;
      oldval=MovingBuffer[i];
      while(k>=i)
        {
         newres=CCIBuffer[k]-oldval;   //Refers to CCI !
         sum+=newres*newres;
         k--;
        }
      deviation=BandsDeviations*MathSqrt(sum/BandsPeriod);
      UpperBuffer[i]=oldval+deviation;
      LowerBuffer[i]=oldval-deviation;
      i--;
     }
//----
   return(0);
   
     }}
     
 void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}
//+------------------------------------------------------------------+